#!/usr/bin/env python

"""Tests for `cloudformation_helper` package."""

from click.testing import CliRunner

from cloudformation_helper import cli


def test_get_help():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.cfhelper)
    assert result.exit_code == 0
    assert "Usage: cfhelper [OPTIONS] COMMAND [ARGS]..." in result.output
    help_result = runner.invoke(cli.cfhelper, ["--help"])
    assert help_result.exit_code == 0
    assert "Usage: cfhelper [OPTIONS] COMMAND [ARGS]..." in help_result.output



#!/usr/bin/env python

"""Tests for bad calls to deploy."""

import pytest
import mock
import os

from click.testing import CliRunner

from cloudformation_helper import cli
from cloudformation_helper.utils import aws

HERE = os.path.dirname(os.path.realpath(__file__))
CONFIG_DIR = os.path.join(HERE, "..", "data", "config")


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=False)
def test_stack_wrong_name(mock_aws_stack_exists, mock_boto3):
    """Create a new stack"""
    runner = CliRunner()

    with pytest.raises(
        Exception, match=r"Could not find stack config named 'IamNotAStack'"
    ):
        runner.invoke(
            cli.cfhelper,
            [
                "--config",
                os.path.join(CONFIG_DIR, "valid_multistacks.cfh"),
                "deploy",
                "IamNotAStack",
            ],
            catch_exceptions=False,
        )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=True)
@mock.patch.object(aws, "has_changeset", return_value=True)
def test_update_with_existing_changeset(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Try to update an existing changeset; but a changeset already exists, bail out"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="n\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_not_called()
    mock_boto3.client.return_value.execute_change_set.assert_not_called()


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=True)
@mock.patch.object(aws, "has_changeset", return_value=True)
def test_update_with_existing_changeset_and_continue(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Try to update an existing changeset; but a changeset already exists, delete it and continue"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="y\ny\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=mock.ANY,
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="UPDATE",
    )

    mock_boto3.client.return_value.delete_change_set.assert_called_once_with(
        StackName="MyStackName", ChangeSetName=aws.CHANGESET_NAME
    )
    mock_boto3.client.return_value.execute_change_set.assert_called_once_with(
        StackName="MyStackName", ChangeSetName=aws.CHANGESET_NAME
    )



#!/usr/bin/env python

"""Tests for `cloudformation_helper` config management."""

import mock
import os
import pytest
import yaml

from click.testing import CliRunner

from cloudformation_helper import cli
from cloudformation_helper.commands import deploy

HERE = os.path.dirname(os.path.realpath(__file__))
CONFIG_DIR = os.path.join(HERE, "data", "config")


def test_config_file_does_not_exist():
    """Pass a file path that does not exist"""
    runner = CliRunner()
    with pytest.raises(FileNotFoundError, match=r"No such file or directory"):
        runner.invoke(
            cli.cfhelper,
            ["--config", "not-a-valid-file", "deploy"],
            catch_exceptions=False,
        )


def test_wrong_config_format():
    """Pass a file that has the wrong format"""
    runner = CliRunner()
    with pytest.raises(yaml.parser.ParserError):
        runner.invoke(
            cli.cfhelper,
            ["--config", os.path.join(CONFIG_DIR, "not_valid_yaml.cfh"), "deploy"],
            catch_exceptions=False,
        )


@mock.patch.object(deploy, "deploy_or_update")
def test_valid_multistacks_config(mock_deploy):
    """Pass a file that contains multiple valid stacks"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_multistacks.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )
    mock_deploy.assert_called_once_with("MyStackName", mock.ANY, mock.ANY, mock.ANY)


@mock.patch.object(deploy, "deploy_or_update")
def test_valid_singlestack_config(mock_deploy):
    """Pass a file that contains a single valid stacks"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_singlestack.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )
    mock_deploy.assert_called_once_with("MyStackName", mock.ANY, False, set())


@mock.patch.object(deploy, "deploy_or_update")
def test_bad_capability(mock_deploy):
    """Pass a file that contains an invalid capability"""
    runner = CliRunner()

    with pytest.raises(Exception):
        runner.invoke(
            cli.cfhelper,
            [
                "--config",
                os.path.join(CONFIG_DIR, "valid_singlestack_bad_capability.cfh"),
                "deploy",
                "MyStackAlias",
            ],
            catch_exceptions=False,
        )

    mock_deploy.assert_not_called()


@mock.patch.object(deploy, "deploy_or_update")
def test_with_capability(mock_deploy):
    """Pass a file that contains one valid capability"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_singlestack_with_capability.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )
    mock_deploy.assert_called_once_with(
        "MyStackName", mock.ANY, False, {"CAPABILITY_IAM"}
    )


@mock.patch.object(deploy, "deploy_or_update")
def test_with_multiple_capabilities(mock_deploy):
    """Pass a file that contains multiple valid capabilities"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(
                CONFIG_DIR, "valid_singlestack_with_multiple_capabilities.cfh"
            ),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )
    mock_deploy.assert_called_once_with(
        "MyStackName", mock.ANY, False, {"CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"}
    )



"""Helpers to format/display various information."""

import click


def display_changeset(changeset):
    for change in changeset["Changes"]:
        if change["Type"] != "Resource":
            continue
        action = change["ResourceChange"]["Action"]
        resource_type = change["ResourceChange"]["ResourceType"]
        resource_id = change["ResourceChange"]["LogicalResourceId"]
        click.echo(f"{resource_type} {resource_id} -> {action}")



"""Helpers to interact with the config file."""
import os

import yaml

VALID_CAPABILITIES = [
    "CAPABILITY_IAM",
    "CAPABILITY_NAMED_IAM",
    "CAPABILITY_AUTO_EXPAND",
]


class Config:
    def __init__(self, stacks):
        self.stacks = stacks

    def get_stack(self, name):
        stack = self.stacks.get(name)
        if stack is None:
            raise Exception(f"Could not find stack config named '{name}'")

        return stack


def parse_stack(section, root):
    stack_name = section.get("stack")
    stack_file = section.get("file")
    use_changesets = section.get("use_changesets")
    capabilities = section.get("capabilities", [])

    if not os.path.isabs(stack_file):
        stack_file = os.path.join(root, stack_file)

    if not os.path.exists(stack_file):
        raise Exception(f"Could not find stack file: '{stack_file}'")

    if not isinstance(capabilities, list):
        raise Exception("Wrong format for capabilities, expecting list")

    capabilities = set(capabilities)
    if not capabilities.issubset(VALID_CAPABILITIES):
        raise Exception(
            f"Some capabilities are not valid; valid values are '{VALID_CAPABILITIES}'"
        )

    return stack_name, stack_file, use_changesets, capabilities


def read_config(config_file_name):
    config_file = os.path.abspath(config_file_name)
    root = os.path.dirname(config_file)
    stacks = {}

    # This will raise if something goes wrong, expected
    with open(config_file, "r") as stream:
        raw_config = yaml.safe_load(stream)

    # Find all the stacks and validate them
    for name, section in raw_config.items():
        stack = parse_stack(section, root)
        stacks[name] = stack

    return Config(stacks)



"""Command used to deploy/update a cloudformation stack."""

import click


import cloudformation_helper.utils.aws as aws
import cloudformation_helper.utils.formatter as formatter


def update_using_changeset(stack_name, stack_file, capabilities):
    click.echo("Creating changeset for existing stack")
    aws.create_changeset(stack_name, stack_file, False, capabilities)
    changeset = aws.get_changeset(stack_name)
    formatter.display_changeset(changeset)
    if click.confirm("Execute stack changes? ", default=False):
        aws.execute_changeset(stack_name, False)
    else:
        if click.confirm("Keep pending changes? ", default=False):
            click.echo("Aborted")
        else:
            aws.delete_changeset(stack_name)


def create_using_changeset(stack_name, stack_file, capabilities):
    click.echo("Creating changeset for new stack")
    aws.create_changeset(stack_name, stack_file, True, capabilities)
    changeset = aws.get_changeset(stack_name)
    formatter.display_changeset(changeset)
    if click.confirm("Execute stack changes? ", default=False):
        aws.execute_changeset(stack_name, True)
    else:
        if click.confirm("Keep pending changes? ", default=False):
            click.echo("Aborted")
        else:
            aws.delete_changeset(stack_name)


def deploy_or_update(stack_name, stack_file, use_changesets, capabilities):
    click.echo(f"Processing {stack_name} using {stack_file}")

    if use_changesets and aws.has_changeset(stack_name):
        if click.confirm("Stack already has a changeset; delete it? ", default=False):
            click.echo(f"Deleting changeset for stack {stack_name}")
            aws.delete_changeset(stack_name)
        else:
            click.echo("Aborted")
            return

    if aws.stack_exists(stack_name):
        if use_changesets:
            update_using_changeset(stack_name, stack_file, capabilities)
        else:
            click.echo("Stack already exists, updating it")
            aws.update_stack(stack_name, stack_file, capabilities)
    else:
        if use_changesets:
            create_using_changeset(stack_name, stack_file, capabilities)
        else:
            click.echo("Stack not found, creating new one")
            aws.create_stack(stack_name, stack_file, capabilities)



"""Helpers to access AWS information."""

import botocore
import boto3


CHANGESET_NAME = "cfhelper-changeset"


def get_cloudformation_client():
    return boto3.client("cloudformation")


def stack_exists(stack_name):
    client = get_cloudformation_client()

    try:
        client.describe_stacks(StackName=stack_name)
    except botocore.exceptions.ClientError as error:
        if (
            error.response["Error"]["Code"] == "ValidationError"
            and "does not exist" in error.response["Error"]["Message"]
        ):
            return False
        raise error

    return True


def create_stack(stack_name, stack_file, capabilities):
    client = get_cloudformation_client()
    with open(stack_file) as f:
        template = f.read()

    client.create_stack(
        StackName=stack_name,
        TemplateBody=template,
        Capabilities=sorted(capabilities),
    )
    waiter = client.get_waiter("stack_create_complete")

    waiter.wait(StackName=stack_name, WaiterConfig={"Delay": 15, "MaxAttempts": 120})


def update_stack(stack_name, stack_file, capabilities):
    client = get_cloudformation_client()
    with open(stack_file) as f:
        template = f.read()

    # TODO, handle errors
    # botocore.errorfactory.InsufficientCapabilitiesException: An error occurred (InsufficientCapabilitiesException)
    # when calling the UpdateStack operation: Requires capabilities : [CAPABILITY_IAM]
    # botocore.exceptions.ClientError: An error occurred (ValidationError) when calling the UpdateStack operation: No updates are to be performed.
    client.update_stack(
        StackName=stack_name,
        TemplateBody=template,
        Capabilities=sorted(capabilities),
    )

    waiter = client.get_waiter("stack_update_complete")
    waiter.wait(StackName=stack_name, WaiterConfig={"Delay": 15, "MaxAttempts": 120})


def has_changeset(stack_name):
    try:
        get_changeset(stack_name)
    except botocore.exceptions.ClientError as error:
        if (
            error.response["Error"]["Code"] == "ValidationError"
            and "does not exist" in error.response["Error"]["Message"]
        ):
            return False
        if error.response["Error"]["Code"] == "ChangeSetNotFound":
            return False
        raise error

    return True


def get_changeset(stack_name):
    client = get_cloudformation_client()

    return client.describe_change_set(
        StackName=stack_name,
        ChangeSetName=CHANGESET_NAME,
    )


def create_changeset(stack_name, stack_file, is_creation, capabilities):
    client = get_cloudformation_client()
    changeset_type = "CREATE" if is_creation else "UPDATE"
    with open(stack_file) as f:
        template = f.read()

    client.create_change_set(
        StackName=stack_name,
        TemplateBody=template,
        Capabilities=sorted(capabilities),
        ChangeSetName=CHANGESET_NAME,
        ChangeSetType=changeset_type,
    )

    waiter = client.get_waiter("change_set_create_complete")
    waiter.wait(
        ChangeSetName=CHANGESET_NAME,
        StackName=stack_name,
        WaiterConfig={"Delay": 15, "MaxAttempts": 120},
    )


def execute_changeset(stack_name, is_creation):
    client = get_cloudformation_client()
    client.execute_change_set(
        StackName=stack_name,
        ChangeSetName=CHANGESET_NAME,
    )

    waiter_type = "stack_create_complete" if is_creation else "stack_update_complete"
    waiter = client.get_waiter(waiter_type)
    waiter.wait(StackName=stack_name, WaiterConfig={"Delay": 15, "MaxAttempts": 120})


def delete_changeset(stack_name):
    client = get_cloudformation_client()

    client.delete_change_set(
        StackName=stack_name,
        ChangeSetName=CHANGESET_NAME,
    )



"""Console script for cloudformation_helper."""
import sys
import click

import cloudformation_helper.commands.deploy as deployModule
from cloudformation_helper.utils.config import read_config


@click.group()
@click.option("--config", default="stacks.cfh")
@click.pass_context
def cfhelper(ctx, config):
    ctx.obj = read_config(config)


@cfhelper.command()
@click.argument("stack_alias")
@click.pass_obj
def deploy(config, stack_alias):
    stack_name, stack_file, use_changesets, capabilities = config.get_stack(stack_alias)
    deployModule.deploy_or_update(stack_name, stack_file, use_changesets, capabilities)


def run():
    sys.exit(cfhelper(auto_envvar_prefix="CFHELPER"))  # pragma: no cover


if __name__ == "__main__":
    run()



#!/usr/bin/env python

"""Tests for valid calls to deploy."""

import mock
import os

from click.testing import CliRunner

from cloudformation_helper import cli
from cloudformation_helper.utils import aws

HERE = os.path.dirname(os.path.realpath(__file__))
CONFIG_DIR = os.path.join(HERE, "..", "data", "config")


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=False)
def test_create(mock_aws_stack_exists, mock_boto3):
    """Create a new stack"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_multistacks.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )
    mock_boto3.client.return_value.create_stack.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=True)
def test_update(mock_aws_stack_exists, mock_boto3):
    """Update an existing stack"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_multistacks.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )
    mock_boto3.client.return_value.update_stack.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=False)
@mock.patch.object(aws, "has_changeset", return_value=False)
def test_create_with_changeset(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Create a new stack using changesets"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="y\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="CREATE",
    )

    mock_boto3.client.return_value.execute_change_set.assert_called_once_with(
        StackName="MyStackName",
        ChangeSetName=aws.CHANGESET_NAME,
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=False)
@mock.patch.object(aws, "has_changeset", return_value=False)
def test_create_with_changeset_no_execute(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Create a new stack using changesets, but don't execute"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="n\ny\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="CREATE",
    )

    mock_boto3.client.return_value.execute_change_set.assert_not_called()
    mock_boto3.client.return_value.delete_change_set.assert_not_called()


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=False)
@mock.patch.object(aws, "has_changeset", return_value=False)
def test_create_with_changeset_no_execute_cleanup(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Create a new stack using changesets, don't execute and delete changeset"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="n\nn\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="CREATE",
    )

    mock_boto3.client.return_value.execute_change_set.assert_not_called()
    mock_boto3.client.return_value.delete_change_set.assert_called_once_with(
        StackName="MyStackName",
        ChangeSetName=aws.CHANGESET_NAME,
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=True)
@mock.patch.object(aws, "has_changeset", return_value=False)
def test_update_with_changeset(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Update an existing stack using changesets"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="y\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="UPDATE",
    )

    mock_boto3.client.return_value.execute_change_set.assert_called_once_with(
        StackName="MyStackName",
        ChangeSetName=aws.CHANGESET_NAME,
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=True)
@mock.patch.object(aws, "has_changeset", return_value=False)
def test_update_with_changeset_no_execute(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Update an existing stack using changesets but don't execute"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="n\ny\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="UPDATE",
    )

    mock_boto3.client.return_value.execute_change_set.assert_not_called()
    mock_boto3.client.return_value.delete_change_set.assert_not_called()


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=True)
@mock.patch.object(aws, "has_changeset", return_value=False)
def test_update_with_changeset_no_execute_cleanup(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Update an existing stack using changesets, don't execute but delete changeset"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(CONFIG_DIR, "valid_with_changeset.cfh"),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="n\nn\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=[],
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="UPDATE",
    )

    mock_boto3.client.return_value.execute_change_set.assert_not_called()
    mock_boto3.client.return_value.delete_change_set.assert_called_once_with(
        StackName="MyStackName",
        ChangeSetName=aws.CHANGESET_NAME,
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=False)
def test_create_with_capabilities(mock_aws_stack_exists, mock_boto3):
    """Create a new stack"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(
                CONFIG_DIR, "valid_singlestack_with_multiple_capabilities.cfh"
            ),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )

    mock_boto3.client.return_value.create_stack.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=True)
def test_update_with_capabilities(mock_aws_stack_exists, mock_boto3):
    """Update an existing stack"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(
                CONFIG_DIR, "valid_singlestack_with_multiple_capabilities.cfh"
            ),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
    )

    mock_boto3.client.return_value.update_stack.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
    )


@mock.patch("cloudformation_helper.utils.aws.boto3")
@mock.patch.object(aws, "stack_exists", return_value=False)
@mock.patch.object(aws, "has_changeset", return_value=False)
def test_create_with_changeset_and_capabilities(
    mock_aws_has_changeset, mock_aws_stack_exists, mock_boto3
):
    """Create a new stack using changesets"""
    runner = CliRunner()

    runner.invoke(
        cli.cfhelper,
        [
            "--config",
            os.path.join(
                CONFIG_DIR, "valid_singlestack_capabilities_and_changeset.cfh"
            ),
            "deploy",
            "MyStackAlias",
        ],
        catch_exceptions=False,
        input="y\n",
    )

    mock_boto3.client.return_value.create_change_set.assert_called_once_with(
        StackName="MyStackName",
        TemplateBody=mock.ANY,
        Capabilities=["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
        ChangeSetName=aws.CHANGESET_NAME,
        ChangeSetType="CREATE",
    )

    mock_boto3.client.return_value.execute_change_set.assert_called_once_with(
        StackName="MyStackName",
        ChangeSetName=aws.CHANGESET_NAME,
    )
